@@ -1,7 +1,7 @@
 /*
     Qalculate    
 
-    Copyright (C) 2003  Niklas Knutsson (nq@altern.org)
+    Copyright (C) 2003-2006  Niklas Knutsson (nq@altern.org)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -98,7 +98,7 @@ DECLARE_BUILTIN_FUNCTION(LimitsFunction)
 DECLARE_BUILTIN_FUNCTION(RankFunction)
 DECLARE_BUILTIN_FUNCTION(SortFunction)
 DECLARE_BUILTIN_FUNCTION(ComponentFunction)
-DECLARE_BUILTIN_FUNCTION_PI(ComponentsFunction)
+DECLARE_BUILTIN_FUNCTION_PI(DimensionFunction)
 
 DECLARE_BUILTIN_FUNCTION(MatrixFunction)
 DECLARE_BUILTIN_FUNCTION(MergeVectorsFunction)
@@ -190,6 +190,10 @@ DECLARE_BUILTIN_FUNCTION(MaxFunction)
 DECLARE_BUILTIN_FUNCTION(ModeFunction)
 DECLARE_BUILTIN_FUNCTION_RPI(RandFunction)
 
+DECLARE_BUILTIN_FUNCTION(ISODateFunction)
+DECLARE_BUILTIN_FUNCTION(LocalDateFunction)
+DECLARE_BUILTIN_FUNCTION(TimestampFunction)
+DECLARE_BUILTIN_FUNCTION(TimestampToDateFunction)
 DECLARE_BUILTIN_FUNCTION(DaysFunction)
 DECLARE_BUILTIN_FUNCTION(YearFracFunction)
 DECLARE_BUILTIN_FUNCTION(WeekFunction)
@@ -233,6 +237,9 @@ DECLARE_BUILTIN_FUNCTION(SaveFunction)
 DECLARE_BUILTIN_FUNCTION(LoadFunction)
 DECLARE_BUILTIN_FUNCTION(ExportFunction)
 
+DECLARE_BUILTIN_FUNCTION(RegisterFunction)
+DECLARE_BUILTIN_FUNCTION(StackFunction)
+
 DECLARE_BUILTIN_FUNCTION(DeriveFunction)
 DECLARE_BUILTIN_FUNCTION(IntegrateFunction)
 DECLARE_BUILTIN_FUNCTION(SolveFunction)
