@@ -1,7 +1,7 @@
 /*
     Qalculate    
 
-    Copyright (C) 2003  Niklas Knutsson (nq@altern.org)
+    Copyright (C) 2003-2006  Niklas Knutsson (nq@altern.org)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -464,15 +464,25 @@ bool MathFunction::testArguments(MathStructure &vargs) {
 MathStructure MathFunction::calculate(MathStructure &vargs, const EvaluationOptions &eo) {
 	int itmp = vargs.size();
 	if(testArgumentCount(itmp)) {
-		while(itmp < maxargs()) {
-			vargs.addItem(CALCULATOR->parse(default_values[itmp - minargs()]));
-			itmp++;
-		}
+		appendDefaultValues(vargs);
 		MathStructure mstruct;
-		if(!testArguments(vargs) || calculate(mstruct, vargs, eo) < 1) {
+		int ret = 0;
+		if(!testArguments(vargs) || (ret = calculate(mstruct, vargs, eo)) < 1) {
+			if(ret < 0) {
+				ret = -ret;
+				if(maxargs() > 0 && ret > maxargs()) {
+					if(mstruct.isVector()) {
+						for(size_t arg_i = 0; arg_i < vargs.size() && arg_i < mstruct.size(); arg_i++) {
+							vargs.setChild(mstruct[arg_i], arg_i + 1);
+						}
+					}
+				} else if(ret <= (int) vargs.size()) {
+					vargs.setChild(mstruct, ret);
+				}
+			}
 			return createFunctionMathStructureFromVArgs(vargs);
 		}
-		if(precision() < 0) mstruct.setPrecision(precision());
+		if(precision() > 0 && precision() < mstruct.precision()) mstruct.setPrecision(precision());
 		if(isApproximate()) mstruct.setApproximate();
 		return mstruct;
 	} else {
@@ -494,6 +504,21 @@ const string &MathFunction::getDefaultValue(size_t arg_) const {
 	}
 	return empty_string;
 }
+void MathFunction::appendDefaultValues(MathStructure &vargs) {
+	if((int) vargs.size() < minargs()) return;
+	while((int) vargs.size() < maxargs()) {
+		Argument *arg = getArgumentDefinition(vargs.size() + 1);
+		if(arg) {
+			MathStructure *mstruct = new MathStructure();
+			arg->parse(mstruct, default_values[vargs.size() - minargs()]);
+			vargs.addChild_nocopy(mstruct);
+		} else {
+			MathStructure *mstruct = new MathStructure();
+			CALCULATOR->parse(mstruct, default_values[vargs.size() - minargs()]);
+			vargs.addChild_nocopy(mstruct);
+		}
+	}
+}
 int MathFunction::stringArgs(const string &argstr, vector<string> &svargs) {
 	svargs.clear();
 	int start_pos = 0;
@@ -583,8 +608,8 @@ MathStructure MathFunction::produceVector(const MathStructure &vargs, int begin,
 		end = vargs.size();
 	}
 	if(begin == 1 && vargs.size() == 1) {
-		if(vargs.getComponent(1)->isVector()) {
-			return *vargs.getComponent(1);
+		if(vargs[0].isVector()) {
+			return vargs[0];
 		} else {
 			return vargs;
 		}
@@ -624,28 +649,28 @@ bool MathFunction::representsUndefined(const MathStructure&) const {return false
 bool MathFunction::representsBoolean(const MathStructure&) const {return false;}
 bool MathFunction::representsNonMatrix(const MathStructure &vargs) const {return representsNumber(vargs, true);}
 
-UserFunction::UserFunction(string cat_, string name_, string eq_, bool is_local, int argc_, string title_, string descr_, int max_argc_, bool is_active) : MathFunction(name_, argc_, max_argc_, cat_, title_, descr_, is_active) {
+UserFunction::UserFunction(string cat_, string name_, string formula_, bool is_local, int argc_, string title_, string descr_, int max_argc_, bool is_active) : MathFunction(name_, argc_, max_argc_, cat_, title_, descr_, is_active) {
 	b_local = is_local;
 	b_builtin = false;
-	setEquation(eq_, argc_, max_argc_);
+	setFormula(formula_, argc_, max_argc_);
 	setChanged(false);
 }
 UserFunction::UserFunction(const UserFunction *function) {
 	set(function);
 }
-string UserFunction::equation() const {
-	return eq;
+string UserFunction::formula() const {
+	return sformula;
 }
-string UserFunction::internalEquation() const {
-	return eq_calc;
+string UserFunction::internalFormula() const {
+	return sformula_calc;
 }
 ExpressionItem *UserFunction::copy() const {
 	return new UserFunction(this);
 }
 void UserFunction::set(const ExpressionItem *item) {
 	if(item->type() == TYPE_FUNCTION && item->subtype() == SUBTYPE_USER_FUNCTION) {
-		eq = ((UserFunction*) item)->equation();
-		eq_calc = ((UserFunction*) item)->internalEquation();
+		sformula = ((UserFunction*) item)->formula();
+		sformula_calc = ((UserFunction*) item)->internalFormula();
 		v_subs.clear();
 		v_precalculate.clear();
 		for(size_t i = 1; i <= ((UserFunction*) item)->countSubfunctions(); i++) {
@@ -663,7 +688,7 @@ int UserFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 
 	ParseOptions po;
 	if(args() != 0) {
-		string stmp = eq_calc;
+		string stmp = sformula_calc;
 		string svar;
 		string v_str, w_str;
 		vector<string> v_strs;
@@ -842,16 +867,22 @@ int UserFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 		if(precision() > 0) mstruct.setPrecision(precision());
 		if(isApproximate()) mstruct.setApproximate();
 	} else {
-		CALCULATOR->parse(&mstruct, eq_calc, po);
+		CALCULATOR->parse(&mstruct, sformula_calc, po);
 		if(precision() > 0) mstruct.setPrecision(precision());
 		if(isApproximate()) mstruct.setApproximate();
 	}
 	return 1;
 }
-void UserFunction::setEquation(string new_eq, int argc_, int max_argc_) {
+void UserFunction::setFormula(string new_formula, int argc_, int max_argc_) {
 	setChanged(true);
-	eq = new_eq;
+	sformula = new_formula;
 	default_values.clear();
+	if(sformula.empty() && v_subs.empty()) {
+		sformula_calc = new_formula;
+		argc = 0;
+		max_argc = 0;
+		return;
+	}
 	if(argc_ < 0) {
 		argc_ = 0, max_argc_ = 0;
 		string svar, svar_o, svar_v;
@@ -859,7 +890,7 @@ void UserFunction::setEquation(string new_eq, int argc_, int max_argc_) {
 		size_t i3 = 0, i4 = 0, i5 = 0;
 		size_t i2 = 0;
 		for(int i = 0; i < 26; i++) {
-			begin_loop_in_set_equation:
+			begin_loop_in_set_formula:
 			i4 = 0; i5 = 0;
 			svar = '\\';
 			svar_o = '\\';
@@ -872,16 +903,16 @@ void UserFunction::setEquation(string new_eq, int argc_, int max_argc_) {
 			else
 				svar_o += 'X' + i;
 				
-			before_find_in_set_equation:
-			if(i < 24 && (i2 = new_eq.find(svar_o, i4)) != string::npos) {
-				if(i2 > 0 && new_eq[i2 - 1] == '\\') {
+			before_find_in_set_formula:
+			if(i < 24 && (i2 = new_formula.find(svar_o, i4)) != string::npos) {
+				if(i2 > 0 && new_formula[i2 - 1] == '\\') {
 					i4 = i2 + 2;
-					goto before_find_in_set_equation;
+					goto before_find_in_set_formula;
 				}				
 				i3 = 0;
-				if(new_eq.length() > i2 + 2 && new_eq[i2 + 2] == ID_WRAP_LEFT_CH) {
-					if((i3 = new_eq.find(ID_WRAP_RIGHT_CH, i2 + 2)) != string::npos) {
-						svar_v = new_eq.substr(i2 + 3, i3 - (i2 + 3));	
+				if(new_formula.length() > i2 + 2 && new_formula[i2 + 2] == ID_WRAP_LEFT_CH) {
+					if((i3 = new_formula.find(ID_WRAP_RIGHT_CH, i2 + 2)) != string::npos) {
+						svar_v = new_formula.substr(i2 + 3, i3 - (i2 + 3));
 						i3 -= i2 + 1;
 					} else i3 = 0;
 				}
@@ -890,12 +921,12 @@ void UserFunction::setEquation(string new_eq, int argc_, int max_argc_) {
 				} else {
 					default_values.push_back("0");
 				}
-				new_eq.replace(i2, 2 + i3, svar);
-				while((i2 = new_eq.find(svar_o, i2 + 1)) != string::npos) {
-					if(i2 > 0 && new_eq[i2 - 1] == '\\') {
+				new_formula.replace(i2, 2 + i3, svar);
+				while((i2 = new_formula.find(svar_o, i2 + 1)) != string::npos) {
+					if(i2 > 0 && new_formula[i2 - 1] == '\\') {
 						i2++;
 					} else {
-						new_eq.replace(i2, 2, svar);
+						new_formula.replace(i2, 2, svar);
 					}
 				}
 				for(size_t sub_i = 0; sub_i < v_subs.size(); sub_i++) {
@@ -909,19 +940,19 @@ void UserFunction::setEquation(string new_eq, int argc_, int max_argc_) {
 					}
 				}
 				optionals = true;
-			} else if((i2 = new_eq.find(svar, i5)) != string::npos) {
-				if(i2 > 0 && new_eq[i2 - 1] == '\\') {
+			} else if((i2 = new_formula.find(svar, i5)) != string::npos) {
+				if(i2 > 0 && new_formula[i2 - 1] == '\\') {
 					i5 = i2 + 2;
-					goto before_find_in_set_equation;
+					goto before_find_in_set_formula;
 				}
 			} else {
 				b = false;
 				for(size_t sub_i = 0; sub_i < v_subs.size(); sub_i++) {
-					before_find_in_vsubs_set_equation:
+					before_find_in_vsubs_set_formula:
 					if(i < 24 && (i2 = v_subs[sub_i].find(svar_o, i4)) != string::npos) {
 						if(i2 > 0 && v_subs[sub_i][i2 - 1] == '\\') {
 							i4 = i2 + 2;
-							goto before_find_in_vsubs_set_equation;
+							goto before_find_in_vsubs_set_formula;
 						}				
 						i3 = 0;
 						if(v_subs[sub_i].length() > i2 + 2 && v_subs[sub_i][i2 + 2] == ID_WRAP_LEFT_CH) {
@@ -948,7 +979,7 @@ void UserFunction::setEquation(string new_eq, int argc_, int max_argc_) {
 					} else if((i2 = v_subs[sub_i].find(svar, i5)) != string::npos) {
 						if(i2 > 0 && v_subs[sub_i][i2 - 1] == '\\') {
 							i5 = i2 + 2;
-							goto before_find_in_vsubs_set_equation;
+							goto before_find_in_vsubs_set_formula;
 						}
 						b = true;
 					}
@@ -956,7 +987,7 @@ void UserFunction::setEquation(string new_eq, int argc_, int max_argc_) {
 				if(!b) {
 					if(i < 24 && !optionals) {
 						i = 24;
-						goto begin_loop_in_set_equation;
+						goto begin_loop_in_set_formula;
 					}
 					break;
 				}
@@ -990,7 +1021,7 @@ void UserFunction::setEquation(string new_eq, int argc_, int max_argc_) {
 		default_values.push_back("0");
 	}
 	if(max_argc_ > 0) default_values.resize(max_argc_ - argc_);
-	eq_calc = new_eq;
+	sformula_calc = new_formula;
 	argc = argc_;
 	max_argc = max_argc_;	
 }
@@ -1147,11 +1178,6 @@ bool Argument::test(MathStructure &value, int index, MathFunction *f, const Eval
 	}
 	return true;
 }
-void Argument::evaluate(MathStructure &mstruct, const EvaluationOptions &eo) const {
-	if(type() != ARGUMENT_TYPE_FREE) {
-		mstruct.eval(eo);
-	}
-}
 MathStructure Argument::parse(const string &str, const ParseOptions &po) const {
 	MathStructure mstruct;
 	parse(&mstruct, str, po);
@@ -1286,18 +1312,21 @@ void Argument::setCustomCondition(string condition) {
 string Argument::getCustomCondition() const {
 	return scondition;
 }
+
 bool Argument::zeroForbidden() const {
 	return !b_zero;
 }
 void Argument::setZeroForbidden(bool forbid_zero) {
 	b_zero = !forbid_zero;
 }
+
 bool Argument::tests() const {
 	return b_test;
 }
 void Argument::setTests(bool does_test) {
 	b_test = does_test;
 }
+
 bool Argument::alerts() const {
 	return b_error;
 }
@@ -1416,7 +1445,7 @@ bool NumberArgument::subtest(MathStructure &value, const EvaluationOptions &eo)
 	if(!value.isNumber()) {
 		value.eval(eo);
 	}
-	if(!value.isNumber() || (!b_rational_number && !value.number().isRational()) || (!b_complex && value.number().isComplex())) {
+	if(!value.isNumber() || (b_rational_number && !value.number().isRational()) || (!b_complex && value.number().isComplex())) {
 		return false;
 	}
 	if(fmin) {
@@ -1430,7 +1459,7 @@ bool NumberArgument::subtest(MathStructure &value, const EvaluationOptions &eo)
 		if(!(cmpr == COMPARISON_RESULT_LESS || (b_incl_max && COMPARISON_IS_EQUAL_OR_LESS(cmpr)))) {
 			return false;
 		}
-	}	
+	}
 	return true;
 }
 int NumberArgument::type() const {
@@ -1671,7 +1700,7 @@ string TextArgument::print() const {return _("text");}
 string TextArgument::subprintlong() const {return _("a text string");}
 bool TextArgument::suggestsQuotes() const {return false;}
 
-DateArgument::DateArgument(string name_, bool does_test, bool does_error) : Argument(name_, does_test, does_error) { b_text = true;}
+DateArgument::DateArgument(string name_, bool does_test, bool does_error) : Argument(name_, does_test, does_error) {b_text = true;}
 DateArgument::DateArgument(const DateArgument *arg) {set(arg); b_text = true;}
 DateArgument::~DateArgument() {}
 bool DateArgument::subtest(MathStructure &value, const EvaluationOptions &eo) const {
@@ -1711,13 +1740,13 @@ bool VectorArgument::subtest(MathStructure &value, const EvaluationOptions &eo)
 	//}
 	if(!value.isVector()) return false;
 	if(b_argloop && subargs.size() > 0) {
-		for(size_t i = 0; i < value.components(); i++) {
+		for(size_t i = 0; i < value.countChildren(); i++) {
 			if(!subargs[i % subargs.size()]->test(value[i], 1, NULL, eo)) {
 				return false;
 			}
 		}
 	} else {
-		for(size_t i = 0; i < subargs.size() && i < value.components(); i++) {
+		for(size_t i = 0; i < subargs.size() && i < value.countChildren(); i++) {
 			if(!subargs[i]->test(value[i], 1, NULL, eo)) {
 				return false;
 			}
@@ -1771,27 +1800,27 @@ Argument *VectorArgument::getArgument(size_t index) const {
 }
 
 MatrixArgument::MatrixArgument(string name_, bool does_test, bool does_error) : Argument(name_, does_test, does_error) {
-	b_sym = false;
+	b_square = false;
 }
 MatrixArgument::MatrixArgument(const MatrixArgument *arg) {
 	set(arg);
-	b_sym = arg->symmetricDemanded();
+	b_square = arg->squareDemanded();
 }
 MatrixArgument::~MatrixArgument() {}
 bool MatrixArgument::subtest(MathStructure &value, const EvaluationOptions &eo) const {
 	//if(!value.isMatrix()) {
 		value.eval(eo);
 	//}
-	return value.isMatrix() && (!b_sym || value.matrixIsSymmetric());
+	return value.isMatrix() && (!b_square || value.matrixIsSquare());
 }
-bool MatrixArgument::symmetricDemanded() const {return b_sym;}
-void MatrixArgument::setSymmetricDemanded(bool sym) {b_sym = sym;}
+bool MatrixArgument::squareDemanded() const {return b_square;}
+void MatrixArgument::setSquareDemanded(bool square) {b_square = square;}
 int MatrixArgument::type() const {return ARGUMENT_TYPE_MATRIX;}
 Argument *MatrixArgument::copy() const {return new MatrixArgument(this);}
 string MatrixArgument::print() const {return _("matrix");}
 string MatrixArgument::subprintlong() const {
-	if(b_sym) {
-		return _("a symmetric matrix");
+	if(b_square) {
+		return _("a square matrix");
 	} else {
 		return _("a matrix");
 	}
