@@ -1,7 +1,7 @@
 /*
     Qalculate    
 
-    Copyright (C) 2003  Niklas Knutsson (nq@altern.org)
+    Copyright (C) 2003-2006  Niklas Knutsson (nq@altern.org)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -69,10 +69,13 @@ bool s2date(string str, GDate *gtime) {
 int week(string str, bool start_sunday) {
 	remove_blank_ends(str);
 	GDate *gtime = g_date_new();
-	bool b;
+	bool b = false;
 	if(str == _("today") || str == "today") {
 		g_date_set_time(gtime, time(NULL));
 		b = true;
+	} else if(str == _("now") || str == "now") {
+		g_date_set_time(gtime, time(NULL));
+		b = true;
 	} else {
 		b = s2date(str, gtime);
 	}
@@ -113,10 +116,13 @@ int week(string str, bool start_sunday) {
 int weekday(string str) {
 	remove_blank_ends(str);
 	GDate *gtime = g_date_new();
-	bool b;
+	bool b = false;
 	if(str == _("today") || str == "today") {
 		g_date_set_time(gtime, time(NULL));
 		b = true;
+	} else if(str == _("now") || str == "now") {
+		g_date_set_time(gtime, time(NULL));
+		b = true;	
 	} else {
 		b = s2date(str, gtime);
 	}
@@ -130,10 +136,13 @@ int weekday(string str) {
 int yearday(string str) {
 	remove_blank_ends(str);
 	GDate *gtime = g_date_new();
-	bool b;
+	bool b = false;
 	if(str == _("today") || str == "today") {
 		g_date_set_time(gtime, time(NULL));
 		b = true;
+	} else if(str == _("now") || str == "now") {
+		g_date_set_time(gtime, time(NULL));
+		b = true;
 	} else {
 		b = s2date(str, gtime);
 	}
@@ -151,6 +160,9 @@ bool s2date(string str, int &year, int &month, int &day) {
 	if(str == _("today") || str == "today") {
 		today(year, month, day);
 		return true;
+	} else if(str == _("now") || str == "now") {
+		today(year, month, day);
+		return true;
 	}
 	GDate *gtime = g_date_new();
 	bool b = s2date(str, gtime);
